/*
 * Copyright  2012 The MorphOS Development Team
 *
 * $Id: GetCatalogStr.c,v 1.1 2012/11/11 10:53:14 itix Exp $
 */

#include <stdio.h>

#include <proto/alib.h>
#include <proto/exec.h>
#include <proto/locale.h>
#include <proto/timer.h>

extern ULONG RangeSeed;

static void bench(const char *catname, size_t min, size_t max)
{
	struct timerequest timer_io;

	timer_io.tr_node.io_Message.mn_ReplyPort = NULL;

	if (OpenDevice("timer.device", UNIT_VBLANK, &timer_io.tr_node, 0) == 0)
	{
		struct Catalog *cat = OpenCatalogA(NULL, catname, NULL);

		if (cat)
		{
			UQUAD starttime, endtime;
			DOUBLE time, res;
			ULONG ticks, i, j, count;

			RangeSeed = 0x12345678;
			count = 0;

			ReadCPUClock(&starttime);

			for (i = 0; i < 1000; i++)
			{
				for (j = min; j < max; j++)
				{
					GetCatalogStr(cat, j, NULL);
					count++;
				}
			}

			ticks = ReadCPUClock(&endtime);

			endtime -= starttime;

			time = (endtime / (DOUBLE)ticks);
			res  = count / time;

			CloseCatalog(cat);

			printf("%f strings/s\n", res);
		}

		CloseDevice(&timer_io.tr_node);
	}
}

static void bench_rand(const char *catname, size_t min, size_t max)
{
	struct timerequest timer_io;

	timer_io.tr_node.io_Message.mn_ReplyPort = NULL;

	if (OpenDevice("timer.device", UNIT_VBLANK, &timer_io.tr_node, 0) == 0)
	{
		struct Catalog *cat = OpenCatalogA(NULL, catname, NULL);

		if (cat)
		{
			UQUAD starttime, endtime;
			DOUBLE time, res;
			ULONG ticks, i, j, count;

			RangeSeed = 0x12345678;
			count = 0;

			ReadCPUClock(&starttime);

			for (i = 0; i < 1000; i++)
			{
				for (j = min; j < max; j++)
				{
					ULONG id = RangeRand(j);

					GetCatalogStr(cat, id, NULL);
					count++;
				}
			}

			ticks = ReadCPUClock(&endtime);

			endtime -= starttime;

			time = (endtime / (DOUBLE)ticks);
			res  = count / time;

			CloseCatalog(cat);

			printf("RangeRand: %f strings/s\n", res);
		}

		CloseDevice(&timer_io.tr_node);
	}
}

int main(int argc, char *argv[])
{
	bench("Ambient.catalog", 0, 1221);
	bench("Ambient.catalog", 0, 5000);
	bench("Ambient.catalog", 0, 15000);
	bench_rand("Ambient.catalog", 0, 1221);
	bench_rand("Ambient.catalog", 0, 5000);
	bench_rand("Ambient.catalog", 0, 15000);
}
